package org.jugtrento.util.javaee6.persistenza;

import java.io.Serializable;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

public abstract class DAOBaseJPA2<T extends OggettoPersistente> implements DAOBase<T>, Serializable {
	protected abstract EntityManager getEntityManager();

	protected abstract Class<T> getClasseDominio();

	@Override
	public long recuperareConteggio() {
		EntityManager em = getEntityManager();
		CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
		Root<T> rt = cq.from(getClasseDominio());
		cq.select(em.getCriteriaBuilder().count(rt));
		Query q = em.createQuery(cq);
		return ((Long) q.getSingleResult()).longValue();
	}

	@Override
	public List<T> recuperareTutti() {
		EntityManager em = getEntityManager();
		CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
		cq.select(cq.from(getClasseDominio()));
		return em.createQuery(cq).getResultList();
	}

	@Override
	public List<T> recuperareAlcuni(int[] intervalo) {
		EntityManager em = getEntityManager();
		CriteriaQuery cq = em.getCriteriaBuilder().createQuery();
		cq.select(cq.from(getClasseDominio()));
		Query q = em.createQuery(cq);
		q.setMaxResults(intervalo[1] - intervalo[0]);
		q.setFirstResult(intervalo[0]);
		return q.getResultList();
	}

	@Override
	public T recuperarePerId(Long id) {
		EntityManager em = getEntityManager();
		return (T) em.find(getClasseDominio(), id);
	}

	@Override
	public void salvare(T objeto) {
		EntityManager em = getEntityManager();
		if (objeto.isPersistente())
			em.merge(objeto);
		else
			em.persist(objeto);
	}

	@Override
	public void cancellare(T objeto) {
		EntityManager em = getEntityManager();
		em.remove(em.merge(objeto));
	}
}
